@@ -20,7 +20,7 @@
 
 """OS scripts related commands"""
 
-# pylint: disable-msg=W0401,W0613,W0614,C0103
+# pylint: disable=W0401,W0613,W0614,C0103
 # W0401: Wildcard import ganeti.cli
 # W0613: Unused argument, since all functions follow the same API
 # W0614: Unused import %s from wildcard import (since we need cli)
@@ -195,6 +195,7 @@ def DiagnoseOS(opts, args):
       else:
         nodes_bad[node_name] = "OS not found"
 
+    # TODO: Shouldn't the global status be calculated by the LU?
     if nodes_valid and not nodes_bad:
       status = "valid"
     elif not nodes_valid and nodes_bad:
@@ -279,17 +280,17 @@ def ModifyOS(opts, args):
 
 
 commands = {
-  'list': (
+  "list": (
     ListOS, ARGS_NONE, [NOHDR_OPT, PRIORITY_OPT],
     "", "Lists all valid operating systems on the cluster"),
-  'diagnose': (
+  "diagnose": (
     DiagnoseOS, ARGS_NONE, [PRIORITY_OPT],
     "", "Diagnose all operating systems"),
-  'info': (
+  "info": (
     ShowOSInfo, [ArgOs()], [PRIORITY_OPT],
     "", "Show detailed information about "
     "operating systems"),
-  'modify': (
+  "modify": (
     ModifyOS, ARGS_ONE_OS,
     [HVLIST_OPT, OSPARAMS_OPT, DRY_RUN_OPT, PRIORITY_OPT,
      HID_OS_OPT, BLK_OS_OPT],
