@@ -36,10 +36,10 @@ from ganeti import utils
 from ganeti import netutils
 
 
-WEEKDAYNAME = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
+WEEKDAYNAME = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
 MONTHNAME = [None,
-             'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
-             'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
+             "Jan", "Feb", "Mar", "Apr", "May", "Jun",
+             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
 
 # Default error message
 DEFAULT_ERROR_CONTENT_TYPE = "text/html"
@@ -178,7 +178,7 @@ class _HttpClientToServerMessageReader(http.HttpMessageReader):
 
     if len(words) == 3:
       [method, path, version] = words
-      if version[:5] != 'HTTP/':
+      if version[:5] != "HTTP/":
         raise http.HttpBadRequest("Bad request version (%r)" % version)
 
       try:
@@ -537,7 +537,7 @@ class HttpServer(http.HttpBase, asyncore.dispatcher):
     """Called for each incoming connection
 
     """
-    # pylint: disable-msg=W0212
+    # pylint: disable=W0212
     (connection, client_addr) = self.socket.accept()
 
     self._CollectChildren(False)
@@ -560,7 +560,7 @@ class HttpServer(http.HttpBase, asyncore.dispatcher):
         utils.ResetTempfileModule()
 
         self.request_executor(self, connection, client_addr)
-      except Exception: # pylint: disable-msg=W0703
+      except Exception: # pylint: disable=W0703
         logging.exception("Error while handling request from %s:%s",
                           client_addr[0], client_addr[1])
         os._exit(1)
