@@ -34,6 +34,7 @@ _KIND_TO_COMMAND = {
   constants.TAG_CLUSTER: "gnt-cluster",
   constants.TAG_NODE: "gnt-node",
   constants.TAG_INSTANCE: "gnt-instance",
+  constants.TAG_NODEGROUP: "gnt-group",
   }
 
 
@@ -71,6 +72,11 @@ def TestNodeTags(node):
   _TestTags(constants.TAG_NODE, node["primary"])
 
 
+def TestGroupTags(group):
+  """gnt-group tags"""
+  _TestTags(constants.TAG_NODEGROUP, group)
+
+
 def TestInstanceTags(instance):
   """gnt-instance tags"""
   _TestTags(constants.TAG_INSTANCE, instance["name"])
