@@ -27,6 +27,7 @@ import stat
 import tempfile
 import unittest
 import logging
+import types
 
 from ganeti import utils
 
@@ -35,16 +36,61 @@ def GetSourceDir():
   return os.environ.get("TOP_SRCDIR", ".")
 
 
+def _SetupLogging(verbose):
+  """Setupup logging infrastructure.
+
+  """
+  fmt = logging.Formatter("%(asctime)s: %(threadName)s"
+                          " %(levelname)s %(message)s")
+
+  if verbose:
+    handler = logging.StreamHandler()
+  else:
+    handler = logging.FileHandler(os.devnull, "a")
+
+  handler.setLevel(logging.NOTSET)
+  handler.setFormatter(fmt)
+
+  root_logger = logging.getLogger("")
+  root_logger.setLevel(logging.NOTSET)
+  root_logger.addHandler(handler)
+
+
 class GanetiTestProgram(unittest.TestProgram):
   def runTests(self):
-    """
+    """Runs all tests.
 
     """
-    logging.basicConfig(filename=os.devnull)
+    _SetupLogging("LOGTOSTDERR" in os.environ)
 
     sys.stderr.write("Running %s\n" % self.progName)
     sys.stderr.flush()
 
+    # Ensure assertions will be evaluated
+    if not __debug__:
+      raise Exception("Not running in debug mode, assertions would not be"
+                      " evaluated")
+
+    # Check again, this time with a real assertion
+    try:
+      assert False
+    except AssertionError:
+      pass
+    else:
+      raise Exception("Assertion not evaluated")
+
+    # The following piece of code is a backport from Python 2.6. Python 2.4/2.5
+    # only accept class instances as test runners. Being able to pass classes
+    # reduces the amount of code necessary for using a custom test runner.
+    # 2.6 and above should use their own code, however.
+    if (self.testRunner and sys.hexversion < 0x2060000 and
+        isinstance(self.testRunner, (type, types.ClassType))):
+      try:
+        self.testRunner = self.testRunner(verbosity=self.verbosity)
+      except TypeError:
+        # didn't accept the verbosity argument
+        self.testRunner = self.testRunner()
+
     return unittest.TestProgram.runTests(self)
 
 
